@@ -31,9 +31,10 @@ def verbose_contents_str(input_string):
     return contents_str(input_string, verbose=True)
 
 
-def size_str(size):
+def size_str(size, include_bytes=False):
     """
     size: number of bytes
+    include_bytes: whether or not to include 'bytes' string in the return value
     Return a human-readable string.
     """
     if size is None:
@@ -41,9 +42,15 @@ def size_str(size):
 
     for unit in ('', 'k', 'm', 'g', 't'):
         if size < 100 and size != int(size):
+            if unit == '' and include_bytes:
+                return '%.1f bytes' % size
             return '%.1f%s' % (size, unit)
+
         if size < 1024:
+            if unit == '' and include_bytes:
+                return '%d bytes' % size
             return '%d%s' % (size, unit)
+
         size /= 1024.0
 
 
