@@ -1,5 +1,7 @@
+import re
 from collections import namedtuple
 from typing import Any, Dict, List, Optional
+from codalab.lib.formatting import size_str
 
 
 class State(object):
@@ -137,6 +139,9 @@ class RunResources(object):
         memory,  # type: int
         disk,  # type: int
         network,  # type: bool
+        tag,  # type: str
+        tag_exclusive,  # type: bool
+        runs_left,  # type: int
     ):
         self.cpus = cpus
         self.gpus = gpus
@@ -145,6 +150,9 @@ class RunResources(object):
         self.memory = memory
         self.disk = disk
         self.network = network
+        self.tag = tag
+        self.tag_exclusive = tag_exclusive
+        self.runs_left = runs_left
 
     @property
     def as_dict(self):
@@ -160,8 +168,97 @@ class RunResources(object):
             memory=int(dct["memory"]),
             disk=int(dct["disk"]),
             network=bool(dct["network"]),
+            tag=dct["tag"],
+            tag_exclusive=bool(dct["tag_exclusive"]),
+            runs_left=dct["runs_left"],
         )
 
+    def has_tag_match(self, run_resources):
+        """
+        :param self: RunResources
+        :param run_resources: RunResources
+
+        Returns True if self and run_resources have matching tags.
+        """
+        tag_match = re.match('(?:tag=)?(.+)', run_resources.tag)
+        if tag_match is not None:
+            return self.tag == tag_match.group(1)
+
+    def dominates(self, run_resources, strict=False):
+        """
+        :param self: RunResources
+        :param run_resources: RunResources
+        :param strict: bool
+
+        Returns True if self's resources dominate resources in run_resources.
+        If strict = True, returns False if self and run_resources are equivalent.
+        """
+        # Check tags.
+        if run_resources.tag:
+            if not self.has_tag_match(run_resources):
+                return False
+        else:
+            if self.tag_exclusive and self.tag:
+                return False
+
+        # Check the number of runs left.
+        if self.runs_left == 0:
+            return False
+
+        # Check all other resources.
+        if strict:
+            if self.cpus <= run_resources.cpus:
+                return False
+            if self.gpus <= run_resources.gpus:
+                return False
+            if self.memory <= run_resources.memory:
+                return False
+            if self.disk <= run_resources.disk:
+                return False
+            return True
+        else:
+            if self.cpus < run_resources.cpus:
+                return False
+            if self.gpus < run_resources.gpus:
+                return False
+            if self.memory < run_resources.memory:
+                return False
+            if self.disk < run_resources.disk:
+                return False
+            return True
+
+    def get_comparison(self, run_resources):
+        """
+        :param self: RunResources
+        :param run_resources: RunResources
+
+        Compares resources in self with resources in run_resources and returns
+        a resource comparison between the two.
+
+        Example Output: '2 CPUs (3 requested), 2g memory (4g requested)'
+        """
+        comparisons = []
+        if run_resources.tag and (run_resources.tag != self.tag):
+            queue = self.tag or 'No'
+            comparisons.append(f'{queue} queue ({run_resources.tag} requested)')
+
+        if self.cpus < run_resources.cpus:
+            comparisons.append(f'{self.cpus} CPUs ({run_resources.cpus} requested)')
+
+        if self.gpus < run_resources.gpus:
+            comparisons.append(f'{self.gpus} GPUs ({run_resources.gpus} requested)')
+
+        if self.memory < run_resources.memory:
+            memory = size_str(self.memory)
+            requested_memory = size_str(run_resources.memory)
+            comparisons.append(f'{memory} memory ({requested_memory} requested)')
+
+        if self.disk < run_resources.disk:
+            disk = size_str(self.disk)
+            requested_disk = size_str(run_resources.disk)
+            comparisons.append(f'{disk} disk ({requested_disk} requested)')
+        return f"[{', '.join(comparisons)}]"
+
 
 class BundleCheckinState(object):
     """
