@@ -152,7 +152,7 @@ class WorkerManager(object):
         if self.args.worker_shared_memory_size_gb:
             command.extend(['--shared-memory-size-gb', str(self.args.worker_shared_memory_size_gb)])
         if self.args.worker_preemptible:
-            command.extend(['--worker-preemptible'])
+            command.extend(['--preemptible'])
 
         return command
 
